# Copyright (c) 2026 Pyry Haulos
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""Proxy container build context (Dockerfile, scripts, Python modules)."""
