# Copyright (c) 2026 Pyry Haulos
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""Static resources bundled into the wheel for PyPI distribution.

Subpackages:

- ``lib._bundled.assets`` — logo SVG
- ``lib._bundled.proxy`` — proxy container build context
"""
